@@ -12,8 +12,10 @@ from bokeh.core.properties import (field, value,
     NumberSpec, ColorSpec, Bool, Int, Float, Complex, String,
     Regex, Seq, List, Dict, Tuple, Instance, Any, Interval, Either,
     Enum, Color, DashPattern, Size, Percent, Angle, AngleSpec, StringSpec,
-    DistanceSpec, FontSizeSpec, Override, Include, MinMaxBounds,
-    DataDistanceSpec, ScreenDistanceSpec)
+    DistanceSpec, FontSize, FontSizeSpec, Override, Include, MinMaxBounds,
+    DataDistanceSpec, ScreenDistanceSpec, ColumnData, UnitsSpec)
+
+from bokeh.core.property.containers import PropertyValueColumnData, PropertyValueDict, PropertyValueList
 
 from bokeh.core.has_props import HasProps
 
@@ -589,12 +591,12 @@ class TestFontSizeSpec(unittest.TestCase):
 
             v = '10%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             v = '10.2%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             f = '_10%s' % unit
@@ -610,12 +612,12 @@ class TestFontSizeSpec(unittest.TestCase):
         for unit in css_units.upper().split("|"):
             v = '10%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             v = '10.2%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             f = '_10%s' % unit
@@ -1190,6 +1192,7 @@ class TestProperties(unittest.TestCase):
         prop = String()
 
         self.assertTrue(prop.is_valid(None))
+
         self.assertFalse(prop.is_valid(False))
         self.assertFalse(prop.is_valid(True))
         self.assertFalse(prop.is_valid(0))
@@ -1198,11 +1201,60 @@ class TestProperties(unittest.TestCase):
         self.assertFalse(prop.is_valid(1.0))
         self.assertFalse(prop.is_valid(1.0+1.0j))
         self.assertTrue(prop.is_valid(""))
+        self.assertTrue(prop.is_valid("6"))
         self.assertFalse(prop.is_valid(()))
         self.assertFalse(prop.is_valid([]))
         self.assertFalse(prop.is_valid({}))
         self.assertFalse(prop.is_valid(Foo()))
 
+    def test_FontSize(self):
+
+        prop = FontSize()
+
+        self.assertTrue(prop.is_valid(None))
+
+        css_units = "%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px"
+
+        for unit in css_units.split("|"):
+            v = '10%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '10.2%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '_10%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+            v = '_10.2%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+        for unit in css_units.upper().split("|"):
+            v = '10%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '10.2%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '_10%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+            v = '_10.2%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+        self.assertFalse(prop.is_valid(False))
+        self.assertFalse(prop.is_valid(True))
+        self.assertFalse(prop.is_valid(0))
+        self.assertFalse(prop.is_valid(1))
+        self.assertFalse(prop.is_valid(0.0))
+        self.assertFalse(prop.is_valid(1.0))
+        self.assertFalse(prop.is_valid(1.0+1.0j))
+        self.assertFalse(prop.is_valid(""))
+        self.assertFalse(prop.is_valid(()))
+        self.assertFalse(prop.is_valid([]))
+        self.assertFalse(prop.is_valid({}))
+        self.assertFalse(prop.is_valid(Foo()))
+
+
     def test_Regex(self):
         with self.assertRaises(TypeError):
             prop = Regex()
@@ -1750,13 +1802,13 @@ bokeh.core.tests.test_properties.Foo5(
 
 def test_field_function():
     assert field("foo") == dict(field="foo")
-    # TODO (bev) would like this to work I think
-    #assert field("foo", transform="junk") == dict(field="foo", transform="junk")
+    assert field("foo", "junk") == dict(field="foo", transform="junk")
+    assert field("foo", transform="junk") == dict(field="foo", transform="junk")
 
 def test_value_function():
     assert value("foo") == dict(value="foo")
-    # TODO (bev) would like this to work I think
-    #assert value("foo", transform="junk") == dict(value="foo", transform="junk")
+    assert value("foo", "junk") == dict(value="foo", transform="junk")
+    assert value("foo", transform="junk") == dict(value="foo", transform="junk")
 
 def test_strict_dataspec_key_values():
     for typ in (NumberSpec, StringSpec, FontSizeSpec, ColorSpec, DataDistanceSpec, ScreenDistanceSpec):
@@ -1767,18 +1819,35 @@ def test_strict_dataspec_key_values():
             f.x = dict(field="foo", units="junk")
 
 def test_dataspec_dict_to_serializable():
-    for typ in (NumberSpec, StringSpec, FontSizeSpec, ColorSpec, DataDistanceSpec, ScreenDistanceSpec):
+    for typ in (NumberSpec, StringSpec, FontSizeSpec, ColorSpec):
         class Foo(HasProps):
             x = typ("x")
         foo = Foo(x=dict(field='foo'))
         props = foo.properties_with_values(include_defaults=False)
-        if typ is DataDistanceSpec:
-            assert props['x']['units'] == 'data'
-        elif typ is ScreenDistanceSpec:
-            assert props['x']['units'] == 'screen'
         assert props['x']['field'] == 'foo'
         assert props['x'] is not foo.x
 
+def test_DataDistanceSpec():
+    assert issubclass(DataDistanceSpec, UnitsSpec)
+    class Foo(HasProps):
+        x = DataDistanceSpec("x")
+    foo = Foo(x=dict(field='foo'))
+    props = foo.properties_with_values(include_defaults=False)
+    assert props['x']['units'] == 'data'
+    assert props['x']['field'] == 'foo'
+    assert props['x'] is not foo.x
+
+def test_ScreenDistanceSpec():
+    assert issubclass(ScreenDistanceSpec, UnitsSpec)
+    class Foo(HasProps):
+        x = ScreenDistanceSpec("x")
+    foo = Foo(x=dict(field='foo'))
+    props = foo.properties_with_values(include_defaults=False)
+    assert props['x']['units'] == 'screen'
+    assert props['x']['field'] == 'foo'
+    assert props['x'] is not foo.x
+
+
 def test_strict_unitspec_key_values():
     class FooUnits(HasProps):
         x = DistanceSpec("x")
@@ -1792,3 +1861,45 @@ def test_strict_unitspec_key_values():
     f.x = dict(field="foo", units="deg")
     with pytest.raises(ValueError):
         f.x = dict(field="foo", units="junk", foo="crap")
+
+def test_Property_wrap():
+    for x in (Bool, Int, Float, Complex, String, Enum, Color,
+              Regex, Seq, Tuple, Instance, Any, Interval, Either,
+              DashPattern, Size, Percent, Angle, MinMaxBounds):
+        for y in (0, 1, 2.3, "foo", None, (), [], {}):
+            r = x.wrap(y)
+            assert r == y
+            assert isinstance(r, type(y))
+
+def test_List_wrap():
+    for y in (0, 1, 2.3, "foo", None, (), {}):
+        r = List.wrap(y)
+        assert r == y
+        assert isinstance(r, type(y))
+    r = List.wrap([1,2,3])
+    assert r == [1,2,3]
+    assert isinstance(r, PropertyValueList)
+    r2 = List.wrap(r)
+    assert r is r2
+
+def test_Dict_wrap():
+    for y in (0, 1, 2.3, "foo", None, (), []):
+        r = Dict.wrap(y)
+        assert r == y
+        assert isinstance(r, type(y))
+    r = Dict.wrap(dict(a=1, b=2))
+    assert r == dict(a=1, b=2)
+    assert isinstance(r, PropertyValueDict)
+    r2 = Dict.wrap(r)
+    assert r is r2
+
+def test_ColumnData_wrap():
+    for y in (0, 1, 2.3, "foo", None, (), []):
+        r = ColumnData.wrap(y)
+        assert r == y
+        assert isinstance(r, type(y))
+    r = ColumnData.wrap(dict(a=1, b=2))
+    assert r == dict(a=1, b=2)
+    assert isinstance(r, PropertyValueColumnData)
+    r2 = ColumnData.wrap(r)
+    assert r is r2
